import os
import sqlite3

from quotly.Quote import Quote


class Quotly:
    def __init__(self):
        if not os.path.exists('data'):
            os.mkdir('data')

        self.conn = sqlite3.connect('data/quotes.db')
        self.initialize()

        print('> Quotly successfully started! Let the quoting begin!')

    def initialize(self):
        cur = self.conn.cursor()

        cur.execute("create table if not exists quotes "
                    "(id integer primary key autoincrement, quote text not null , author text)")

        self.conn.commit()

    def store_quote(self, quote, *targets):
        cur = self.conn.cursor()

        t = []
        if len(targets[0]) > 0:
            for targ in targets[0]:
                t.append(targ)

        t.extend([''] * (1 - len(t)))
        cur.execute(
            "insert into quotes (quote, author) "
            "values (?, ?)", (quote, t[0]))

        self.conn.commit()

        # fetch latest quote
        cur.execute("select quote, author "
                    "from quotes order by id desc limit 1")

        q = Quote(cur.fetchone())
        print('> Quote stored: {0}'.format(q))
        return q

    def fetch_quote(self):
        cur = self.conn.cursor()

        cur.execute("select quote, author "
                    "from quotes order by random() limit 1")

        return Quote(cur.fetchone())

    def fetch_quote_with_targets(self, *targets):
        cur = self.conn.cursor()

        t = list(targets)
        t.extend([''] * (1 - len(targets)))
        cur.execute("select quote, author "
                    "from quotes "
                    "where author == (?) "
                    "order by random() limit 1", t[0])

        return Quote(cur.fetchone())

    def __del__(self):
        self.conn.close()



import os
import argparse

from quotly.QuotyBot import quoty_bot

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description='quotly command line interface')
    parser.add_argument('-r', '--run', action='store_true', help='Start quotly')
    parser.add_argument('-t', '--token', type=str, default=None, help='Set discord bot token')

    args = parser.parse_args()

    token = None
    if args.token is not None:
        with open('.env', 'w') as cfg:
            cfg.write('TOKEN={0}'.format(token))

        print('> Token successfully stored!')
        exit(0)

    if args.run:

        # using token from .env file
        if os.path.exists('.env') and os.getenv('TOKEN') and token is None:
            token = os.getenv('TOKEN')

        if token is not None:
            quoty_bot.run(token)
        else:
            print('> No discord bot token found!')



class Quote:
    def __init__(self, *values):
        self.quote = None
        self.targets = []

        if values is not None and values[0] is not None:
            self.quote = values[0][0]

            for t in values[0][1:]:
                if t is not None and len(t) > 0:
                    self.targets.append(t)

    @staticmethod
    def from_tuple(quote, *targets):
        return Quote((quote, targets))

    def __str__(self):
        if self.quote is not None:
            return '"{0}" -{1}'.format(self.quote, self.targets[0] if len(self.targets) > 0 else 'Unknown')

        return 'Sorry! Found no suitable quote for your input.'



from discord import DMChannel
from dotenv import load_dotenv, find_dotenv
from discord.ext import commands
from quotly.Quotly import Quotly

load_dotenv(find_dotenv())
quotly = Quotly()
quoty_bot = commands.Bot(command_prefix='!')

cmds = {
    'add_quote': 'quotly-add',
    'get_quote': 'quotly-get'
}


@quoty_bot.command(name=cmds['add_quote'])
async def create(ctx, q, *targets):
    # keep the channel clean
    if not isinstance(ctx.channel, DMChannel):
        await ctx.message.delete()

    # message user that his quote is stored
    await ctx.author.send('Quote added: {0}'.format((quotly.store_quote(q, targets))))


@quoty_bot.command(name=cmds['get_quote'])
async def quote(ctx, *targets):
    if len(targets) > 0:
        await ctx.send(quotly.fetch_quote_with_targets(targets))
    else:
        await ctx.send(quotly.fetch_quote())
